@@ -1,4 +1,4 @@
-require 'puppet/error'
+require_relative '../../puppet/error'
 
 module Puppet::Indirector
   class ValidationError < Puppet::Error; end
