@@ -1,4 +1,4 @@
-require 'puppet/indirector/terminus'
+require_relative '../../puppet/indirector/terminus'
 
 # A none terminus type, meant to always return nil
 class Puppet::Indirector::None < Puppet::Indirector::Terminus
