@@ -1,4 +1,4 @@
-require 'puppet/util/logging'
+require_relative '../../puppet/util/logging'
 
 class Puppet::Module
   class Task
@@ -45,13 +45,17 @@ class Puppet::Module
     end
 
     FORBIDDEN_EXTENSIONS = %w{.conf .md}
-    MOUNTS = %w[lib files tasks]
+    MOUNTS = %w[files lib scripts tasks]
 
     def self.is_task_name?(name)
       return true if name =~ /^[a-z][a-z0-9_]*$/
       return false
     end
 
+    def self.is_tasks_file?(path)
+      File.file?(path) && is_tasks_filename?(path)
+    end
+
     # Determine whether a file has a legal name for either a task's executable or metadata file.
     def self.is_tasks_filename?(path)
       name_less_extension = File.basename(path, '.*')
@@ -76,11 +80,23 @@ class Puppet::Module
     # Find task's required lib files and retrieve paths for both 'files' and 'implementation:files' metadata keys
     def self.find_extra_files(metadata, envname = nil)
       return [] if metadata.nil?
+      
+      files = metadata.fetch('files', [])
+      unless files.is_a?(Array)
+        msg = _("The 'files' task metadata expects an array, got %{files}.") % {files: files}
+        raise InvalidMetadata.new(msg, 'puppet.tasks/invalid-metadata')
+      end
+      impl_files = metadata.fetch('implementations', []).flat_map do |impl| 
+        file_array = impl.fetch('files', [])
+        unless file_array.is_a?(Array)
+          msg = _("The 'files' task metadata expects an array, got %{files}.") % {files: file_array}
+          raise InvalidMetadata.new(msg, 'puppet.tasks/invalid-metadata')
+        end
+        file_array
+      end
 
-      files = metadata.fetch('files', []) +
-        metadata.fetch('implementations', []).flat_map { |impl| impl.fetch('files', []) }
-
-      files.uniq.flat_map do |file|
+      combined_files = files + impl_files
+      combined_files.uniq.flat_map do |file|
         module_name, mount, endpath = file.split("/", 3)
         # If there's a mount directory with no trailing slash this will be nil
         # We want it to be empty to construct a path
@@ -142,6 +158,10 @@ class Puppet::Module
         end
 
         implementations = metadata['implementations'].map do |impl|
+          unless impl['requirements'].is_a?(Array) || impl['requirements'].nil?
+            msg = _("Task metadata for task %{name} does not specify requirements as an array" % { name: name })
+            raise InvalidMetadata.new(msg, 'puppet.tasks/invalid-metadata')
+          end
           path = executables.find { |real_impl| File.basename(real_impl) == impl['name'] }
           unless path
             msg = _("Task metadata for task %{name} specifies missing implementation %{implementation}" % { name: name, implementation: impl['name'] })
@@ -184,7 +204,7 @@ class Puppet::Module
 
     def self.tasks_in_module(pup_module)
       task_files = Dir.glob(File.join(pup_module.tasks_directory, '*'))
-        .keep_if { |f| is_tasks_filename?(f) }
+        .keep_if { |f| is_tasks_file?(f) }
 
       module_executables = task_files.reject(&method(:is_tasks_metadata_filename?)).map.to_a
 
